/*

Copyright (c) 2008 Esria, Inc. (esria.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/
package com.flextoolbox.flint.actions
{
	import com.yahoo.astra.utils.PointUtil;
	
	import flash.geom.Point;
	
	import org.flintparticles.actions.Action;
	import org.flintparticles.emitters.Emitter;
	import org.flintparticles.particles.Particle;

	/**
	 * The BoundingCircle action confines each particle to a circle. The
	 * particle bounces back off the side of the box when it reaches the edge.
	 * The bounce treats the particle as a circular body and displays no loss of
	 * energy in the collision.
	 * 
	 * @author Josh Tynjala
	 */
	public class BoundingCircle extends Action
	{
		
	//----------------------------------
	//  Constructor
	//----------------------------------
	
		/**
		 * The constructor creates a BoundingCircle action for use by an
		 * emitter. To add a BoundingCircle to all particles created by an
		 * emitter, use the emitter's addAction() method.
		 * 
		 * @see org.flintparticles.emitters.Emitter#addAction()
		 * @see org.flintparticles.actions.BoundingBox
		 * 
		 * @param center	The center of the bounding circle.
		 * @param radius	The radius of the bounding ciricle.
		 */
		public function BoundingCircle(center:Point, radius:Number)
		{
			super();
			this.center = center.clone();
			this.radius = radius;
		}
		
	//----------------------------------
	//  Properties
	//----------------------------------
		
		/**
		 * The center point of the bounding circle.
		 */
		public var center:Point;
		
		/**
		 * The radius of the bounding circle, in pixels.
		 */
		public var radius:Number;
		
	//----------------------------------
	//  Public Methods
	//----------------------------------

		/**
		 * @inheritDoc
		 * 
		 * <p>Returns a value of -20, so that the BoundingBox executes after all movement has occured.</p>
		 */
		override public function getDefaultPriority():Number
		{
			return -20;
		}

		/**
		 * @inheritDoc
		 */
		override public function update(emitter:Emitter, particle:Particle, time:Number) :void
		{
			var halfWidth:Number;
			var halfHeight:Number;
			if( particle.image )
			{
				halfWidth = particle.scale * particle.image.width * 0.5;
				halfHeight = particle.scale * particle.image.height * 0.5;
			}
			else
			{
				halfWidth = halfHeight = 0;
			}
			
			//just assume that the particle is a circle
			var particleRadius:Number = Math.max(halfWidth, halfHeight);
			
			var particlePosition:Point = new Point(particle.x, particle.y)
			var angle:Number = PointUtil.angle(center, particlePosition);
			particlePosition = particlePosition.subtract(Point.polar(particleRadius, angle));
			
			//if we've overshot the edge, we need to bounce off
			var radialOvershot:Number = Point.distance(center, particlePosition) - this.radius;
			if(radialOvershot <= 0)
			{
				return;
			}
			
			//update position
			var overVector:Point = Point.polar(radialOvershot, angle);
			particle.x += overVector.x;
			particle.y += overVector.y;
			
			//update with new velocity
			var velocity:Point = new Point(particle.velX, particle.velY); //current velocity
			var particleAngle:Number = PointUtil.angle(new Point(0, 0), velocity); //current angle of the particle
			angle = angle - Math.PI; //angle of the circle's edge
			particleAngle = 2 * angle - particleAngle; //new angle of the particle
			velocity = Point.polar(velocity.length, particleAngle); //new velocity
			
			particle.velX = velocity.x;
			particle.velY = velocity.y;
		}
		
	}
}