/*

Copyright (c) 2008 Esria, Inc. (esria.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/
package com.flextoolbox.flint.actions
{
	import org.flintparticles.actions.Action;
	import org.flintparticles.emitters.Emitter;
	import org.flintparticles.particles.Particle;

	/**
	 * An Action with a sub-Action that will only be updated when a specific
	 * condition is met, as specified by the <code>condition</code> property.
	 * 
	 * @author Josh Tynjala
	 */
	public class TargetedAction extends Action
	{
		
	//----------------------------------
	//  Constructor
	//----------------------------------
	
		/**
		 * Constructor.
		 * 
		 * @param action		An Action that should only be updated when a
		 * 						specific condition is met.
		 * @param condition		A function that is called to determine if the action
		 * 						should be updated.
		 */
		public function TargetedAction(action:Action, condition:Function = null)
		{
			super();
			this.action = action;
			this.condition = condition;
		}
		
	//----------------------------------
	//  Properties
	//----------------------------------
		
		/**
		 * An Action that will only be updated when a certain condition is met.
		 */
		public var action:Action;
		
		/**
		 * This function is called every time the TargetedAction is updated. If
		 * the return value is <code>true</code> the sub-action's <code>update()</code>
		 * method is called with the specified parameters. If the return value
		 * is false, the particle is not updated. The condition function has the
		 * following signature:
		 * 
		 * <pre>function(emitter:Emitter, particle:Particle, time:Number):Boolean</pre>
		 * 
		 * <p>If no condition is specified, the action will always be updated.</p>
		 */
		public var condition:Function;
		
	//----------------------------------
	//  Public Methods
	//----------------------------------
		
		/**
		 * @private
		 */
		override public function addedToEmitter(emitter:Emitter):void
		{
			this.action.addedToEmitter(emitter);
		}
		
		/**
		 * @private
		 */
		override public function getDefaultPriority():Number
		{
			return this.action.getDefaultPriority();
		}
		
		/**
		 * @private
		 */
		override public function update(emitter:Emitter, particle:Particle, time:Number):void
		{
			if(this.condition == null || this.condition(emitter, particle, time))
			{
				this.action.update(emitter, particle, time);
			}
		}
		
	}
}