/*

Copyright (c) 2008 Esria, Inc. (esria.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/
package com.flextoolbox.roktok.views
{
	import com.flextoolbox.roktok.vo.Message;
	
	import flash.display.Shape;
	import flash.events.MouseEvent;
	import flash.filters.DropShadowFilter;
	
	import mx.controls.Image;
	import mx.core.UIComponent;
	import mx.effects.Tween;
	import mx.utils.ColorUtil;

	/**
	 * A particle display object for a Message VO. Displays an image for the message
	 * and the color associated with the Message's Artist VO.
	 * 
	 * @author Josh Tynjala
	 */
	public class MessageParticle extends UIComponent
	{	
		
	//----------------------------------
	//  Constructor
	//----------------------------------
	
		/**
		 * Constructor.
		 */
		public function MessageParticle()
		{
			super();
			this.addEventListener(MouseEvent.CLICK, clickHandler);
			this.addEventListener(MouseEvent.ROLL_OVER, rollOverHandler, false, 10);
		}
		
	//----------------------------------
	//  Properties
	//----------------------------------
		
		/**
		 * @private
		 * The user image.
		 */
		protected var image:Image;
		
		/**
		 * @private
		 * The mask for the user image.
		 */
		protected var imageMask:Shape;
		
		/**
		 * @private
		 * Storage for the message property.
		 */
		private var _message:Message;
		
		/**
		 * The message to display.
		 */
		public function get message():Message
		{
			return this._message;
		}
		
		/**
		 * @private
		 */
		public function set message(value:Message):void
		{
			this._message = value;
			this.invalidateProperties();
			this.invalidateSize();
			this.invalidateDisplayList();
		}
		
		/**
		 * @private
		 * Storage for the activated property.
		 */
		private var _activated:Boolean = false;
		
		/**
		 * If true, the particle's message has been viewed, and its colors will be darkened.
		 */
		public function get activated():Boolean
		{
			return this._activated;
		}
		
		/**
		 * @private
		 */
		public function set activated(value:Boolean):void
		{
			this._activated = value;
			this.invalidateDisplayList();
		}
		
		/**
		 * @private
		 * Flag indicating that the mouse over state has changed.
		 */
		protected var mouseIsOverChanged:Boolean = false;
		
		/**
		 * @private
		 * Storage for the mouseIsOver property.
		 */
		private var _mouseIsOver:Boolean = false;
		
		/**
		 * @private
		 * If true, the user has the mouse over the particle.
		 */
		protected function get mouseIsOver():Boolean
		{
			return this._mouseIsOver;
		}
		
		/**
		 * @private
		 */
		protected function set mouseIsOver(value:Boolean):void
		{
			this._mouseIsOver = value;
			this.mouseIsOverChanged = true;
			this.invalidateSize();
			this.invalidateDisplayList();
		}
		
		/**
		 * @private
		 * The width value the last time the particle redrew.
		 */
		protected var oldWidth:Number = NaN;
		
		/**
		 * @private
		 * The height value the last time the particle redrew.
		 */
		protected var oldHeight:Number = NaN;
		
		/**
		 * @private
		 * A tween the change the dimensions of the particle.
		 */
		protected var sizeTween:Tween;
		
	//----------------------------------
	//  Protected Methods
	//----------------------------------
		
		/**
		 * @private
		 */
		override protected function createChildren():void
		{
			super.createChildren();
			
			if(!this.image)
			{
				this.image = new Image();
				this.image.styleName = "messageUserProfileImage";
				this.addChild(this.image);
			}
			
			if(!this.imageMask)
			{
				this.imageMask = new Shape();
				this.addChild(this.imageMask);
				this.image.mask = this.imageMask;
			}
		}
		
		/**
		 * @private
		 */
		override protected function commitProperties():void
		{
			super.commitProperties();
			
			if(this.message)
			{
				this.image.source = this.message.image;
			}
			else
			{
				this.image.source = null;
			}
		}
		
		/**
		 * @private
		 */
		override protected function measure():void
		{
			super.measure();
			if(this.mouseIsOver)
			{
				this.measuredWidth = this.measuredHeight = Math.max(48, Math.min(this.image.getExplicitOrMeasuredWidth(), this.image.getExplicitOrMeasuredHeight()));
			}
			else
			{
				this.measuredWidth = this.measuredHeight = 30;
			}
		}
		
		/**
		 * @private
		 */
		override protected function updateDisplayList(unscaledWidth:Number, unscaledHeight:Number):void
		{
			super.updateDisplayList(unscaledWidth, unscaledHeight);
			
			if(!this.message)
			{
				this.graphics.clear();
				this.imageMask.graphics.clear();
				return;
			}
			
			this.image.setActualSize(this.image.getExplicitOrMeasuredWidth(), this.image.getExplicitOrMeasuredHeight());
			this.image.move(-this.image.width / 2, -this.image.height / 2);
			this.image.alpha = this.activated ? 0.4 : 1;
			
			if(this.sizeTween)
			{
				this.sizeTween.stop();
				this.sizeTween = null;
			}
			
			if(this.mouseIsOverChanged && !isNaN(this.oldWidth) && !isNaN(this.oldHeight))
			{
				this.drawBackgroundAndMask(this.oldWidth, this.oldHeight);
				this.sizeTween = new Tween(this, [this.oldWidth, this.oldHeight], [unscaledWidth, unscaledHeight], 100, -1, sizeUpdateHandler, sizeEndHandler);
			}
			else
			{
				this.drawBackgroundAndMask(unscaledWidth, unscaledHeight);
			}
			
			this.mouseIsOverChanged = false;
			this.oldWidth = unscaledWidth;
			this.oldHeight = unscaledHeight;
		}
		
		/**
		 * @private
		 * Draws the background color and the mask for the image.
		 */
		protected function drawBackgroundAndMask(w:Number, h:Number):void
		{	
			var radius:Number = Math.min(w, h) / 2;
			
			this.imageMask.graphics.clear();
			this.imageMask.graphics.beginFill(0xff00ff);
			this.imageMask.graphics.drawCircle(0, 0, radius - 4);
			this.imageMask.graphics.endFill();
			
			var fillColor:uint = this.message.artist.color;
			fillColor = this.activated ? ColorUtil.adjustBrightness2(fillColor, -60) : fillColor;
			
			this.graphics.clear();
			this.graphics.beginFill(fillColor);
			this.graphics.drawCircle(0, 0, radius);
			this.graphics.endFill();
		}
		
		/**
		 * @private
		 * Update handler for the size tween.
		 */
		protected function sizeUpdateHandler(value:Object):void
		{
			this.drawBackgroundAndMask(value[0], value[1]);
		}
		
		/**
		 * @private
		 * End handler for the size tween.
		 */
		protected function sizeEndHandler(value:Object):void
		{
			this.sizeUpdateHandler(value);
			this.sizeTween = null;
		}
		
	//----------------------------------
	//  Protected Event Handlers
	//----------------------------------
		
		/**
		 * @private
		 * Updates activated when the user clicks on the particle.
		 */
		protected function clickHandler(event:MouseEvent):void
		{
			this.activated = true;
		}
		
		/**
		 * @private
		 * Adds a drop shadow on roll over to distinguish this particle.
		 */
		protected function rollOverHandler(event:MouseEvent):void
		{
			this.addEventListener(MouseEvent.ROLL_OUT, rollOutHandler);
			this.filters = [new DropShadowFilter(0, 45, 0x000000, 1, 10, 10, 2, 2)];
			this.mouseIsOver = true;
		}
		
		/**
		 * @private
		 * Removes the drop shadow on roll out.
		 */
		protected function rollOutHandler(event:MouseEvent):void
		{
			this.removeEventListener(MouseEvent.ROLL_OUT, rollOutHandler);
			this.filters = [];
			this.mouseIsOver = false;
		}
		
	}
}