/*

Copyright (c) 2008 Esria, Inc. (esria.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/
package com.flextoolbox.roktok.views
{
	import com.flextoolbox.flint.actions.MoveWithExceptions;
	import com.flextoolbox.flint.actions.QuickFade;
	import com.flextoolbox.flint.actions.TargetedAction;
	import com.flextoolbox.flint.counters.DataCounter;
	import com.flextoolbox.flint.renderers.UIComponentRenderer;
	import com.flextoolbox.roktok.vo.Artist;
	import com.flextoolbox.roktok.vo.Message;
	import com.flextoolbox.util.ClassFactoryWithEvents;
	
	import flash.display.DisplayObject;
	import flash.display.DisplayObjectContainer;
	import flash.events.MouseEvent;
	import flash.geom.Point;
	
	import mx.collections.ArrayCollection;
	import mx.core.UIComponent;
	import mx.events.CloseEvent;
	import mx.events.CollectionEvent;
	import mx.events.CollectionEventKind;
	import mx.managers.PopUpManager;
	
	import org.flintparticles.actions.*;
	import org.flintparticles.emitters.Emitter;
	import org.flintparticles.initializers.Lifetime;
	import org.flintparticles.particles.Particle;
	import org.flintparticles.renderers.Renderer;

	/**
	 * Displays particles for messages returned from the community messaging service.
	 * 
	 * @author Josh Tynjala
	 */
	public class ParticleViewer extends UIComponent
	{
		
	//----------------------------------
	//  Constructor
	//----------------------------------
	
		/**
		 * Constructor.
		 */
		public function ParticleViewer()
		{
			super();
		}
		
	//----------------------------------
	//  Properties
	//----------------------------------
		
		/**
		 * @private
		 * The shared data tip for the particles.
		 */
		protected var dataTip:ParticleDataTip;
		
		/**
		 * @private
		 * The particle emitter.
		 */
		protected var emitter:Emitter;
		
		/**
		 * @private
		 * The particle renderer.
		 */
		protected var renderer:Renderer;
		
		/**
		 * @private
		 * The counter used to add more particles.
		 */
		protected var counter:DataCounter;
		
		/**
		 * @private
		 * Actions that may change if the stage size changes.
		 */
		protected var groupActions:Array = [];
		
		/**
		 * @private
		 * Particles will move except when the mouse is over them.
		 */
		protected var moveAction:MoveWithExceptions;
		
		/**
		 * @private
		 * The factory used to create new particles.
		 */
		protected var particleFactory:ClassFactoryWithEvents;
		
		/**
		 * @private
		 * The last particle to be touched by the mouse.
		 */
		protected var lastTouched:DisplayObject;
		
		/**
		 * @private
		 * Flag indicating that the messages property has changed.
		 */
		protected var messagesChanged:Boolean = false;
		
		/**
		 * @private
		 * Storage for the messages property.
		 */
		private var _messages:ArrayCollection;
		
		[Bindable]
		/**
		 * The messages to be displayed as particles.
		 */
		public function get messages():ArrayCollection
		{
			return this._messages;
		}
		
		/**
		 * @private
		 */
		public function set messages(value:ArrayCollection):void
		{
			if(!value)
			{
				value = new ArrayCollection();
			}
			
			if(this._messages)
			{
				this._messages.removeEventListener(CollectionEvent.COLLECTION_CHANGE, messagesChangeHandler);
			}
			
			this._messages = value;
			this.addParticles(this._messages.source.concat());
			
			this._messages.addEventListener(CollectionEvent.COLLECTION_CHANGE, messagesChangeHandler, false, 0, true);
		}
		
		/**
		 * @private
		 * Storage for the artists property.
		 */
		private var _artists:ArrayCollection;
		
		/**
		 * The artists associated with the displayed messages.
		 */
		public function get artists():ArrayCollection
		{
			return this._artists;
		}
		
		/**
		 * @private
		 */
		public function set artists(value:ArrayCollection):void
		{
			this._artists = value;
			this.invalidateDisplayList();
		}
		
		/**
		 * @private
		 * Storage for the sortMode property.
		 */
		private var _sortMode:String = ParticleViewerSortMode.NO_SORTING;
		
		/**
		 * The way that particles are displayed. Applies to groupings. Certain
		 * particles are "attracted" to certain areas of the screen.
		 */
		public function get sortMode():String
		{
			return this._sortMode;
		}
		
		/**
		 * @private
		 */
		public function set sortMode(value:String):void
		{
			this._sortMode = value;
			this.invalidateDisplayList();
		}
		
	//----------------------------------
	//  Protected Methods
	//----------------------------------
		
		/**
		 * @private
		 */
		override protected function createChildren():void
		{
			super.createChildren();
			
			this.renderer = new UIComponentRenderer();
			DisplayObjectContainer(this.renderer).mouseChildren = true;
			this.addChild(DisplayObject(this.renderer));
			
			this.particleFactory = new ClassFactoryWithEvents(MessageParticle);
			this.particleFactory.properties = {buttonMode: true};
			this.particleFactory.events = {click: particleClickHandler, rollOver: particleRollOverHandler};
			
			this.counter = new DataCounter();
			this.counter.maximumPerFrame = 0.25;
			this.counter.targetRenderer = this;
			this.counter.displayObjectFactory = this.particleFactory;
			this.counter.dataField = "message";
			
			this.emitter = new Emitter();
			this.emitter.renderer = this.renderer;
			this.emitter.counter = this.counter;
			
			this.moveAction = new MoveWithExceptions();
			this.emitter.addAction(this.moveAction);
			
			this.emitter.addInitializer(new Lifetime(Number.POSITIVE_INFINITY)); //don't actually die
			this.emitter.addAction(new RandomDrift(100, 100));
			this.emitter.addAction(new MinimumDistance(20, 100));
			this.emitter.addAction(new SpeedLimit(100));
			this.emitter.addAction(new SpeedLimit(60, true));
			this.emitter.addAction(new Age());
			this.emitter.addAction(new QuickFade());
			
			this.emitter.start();
			
			if(!this.dataTip)
			{
				this.dataTip = new ParticleDataTip();
			}
		}
		
		/**
		 * @private
		 */
		override protected function updateDisplayList(unscaledWidth:Number, unscaledHeight:Number):void
		{
			super.updateDisplayList(unscaledWidth, unscaledHeight);
			
			var center:Point = new Point(unscaledWidth / 2, unscaledHeight / 2);
			
			var actionsCount:int = this.groupActions.length;
			for(var i:int = 0; i < actionsCount; i++)
			{
				var action:Action = Action(this.groupActions.shift()); 
				this.emitter.removeAction(action);
			}
			
			switch(this.sortMode)
			{
				case ParticleViewerSortMode.SORT_BY_ARTIST:
				{
					this.createActionsByArtist(center);
					break;
				}
				default:
				{
					var turn:TurnTowardsPoint = new TurnTowardsPoint(center.x, center.y, 0.9);
					this.groupActions.push(turn);
					this.emitter.addAction(turn);
				}
			}
			
			this.particleFactory.properties.x = center.x;
			this.particleFactory.properties.y = center.y;
			
			//used for debugging purposes.
			/*this.graphics.clear();
			this.graphics.lineStyle(1, 0xffffff);
			this.graphics.drawCircle(center.x, center.y, 200);*/
		}
		
		/**
		 * @private
		 * Goes through each artist and sets up actions for particles.
		 */
		protected function createActionsByArtist(center:Point):void
		{
			var artistCount:int = this.artists.length;
			for(var i:int = 0; i < artistCount; i++)
			{
				var artist:Artist = Artist(this.artists[i]);
				var angle:Number = i * (2 * Math.PI / artistCount);
				var position:Point = Point.polar(210, angle).add(center);
				
				this.createTargetedCenterForArtist(position, artist);
			}
		}
		
		/**
		 * @private
		 * Creates a targeted TurnTowardsPoint action for a particular Artist.
		 */
		protected function createTargetedCenterForArtist(position:Point, artist:Artist):void
		{
			var targetedAction:TargetedAction = new TargetedAction(new TurnTowardsPoint(position.x, position.y, 1.7));
			targetedAction.condition = function(emitter:Emitter, particle:Particle, time:Number):Boolean
			{
				return MessageParticle(particle.image).message.artist == artist;
			}
			this.groupActions.push(targetedAction);
			this.emitter.addAction(targetedAction);
		}
		
		/**
		 * @private
		 * Adds new particles to the counter.
		 */
		protected function addParticles(sourceData:Array):void
		{
			var itemCount:int = sourceData.length;
			for(var i:int = 0; i < itemCount; i++)
			{
				var message:Message = Message(sourceData[i]);
				this.counter.addData(message);
			}
		}
		
	//----------------------------------
	//  Protected Event Handlers
	//----------------------------------
		
		/**
		 * @private
		 * When the messages data provider changes, adds the new items to the
		 * counter.
		 */
		protected function messagesChangeHandler(event:CollectionEvent):void
		{
			if(event.kind == CollectionEventKind.ADD)
			{
				this.addParticles(event.items);
			}
		}
		
		/**
		 * @private
		 * When a particle is clicked, shows the MessageDetails window with the
		 * message data. Pauses the emitter until the window closes.
		 */
		protected function particleClickHandler(event:MouseEvent):void
		{
			this.emitter.pause();
			
			var message:Message = MessageParticle(event.currentTarget).message;
			
			var details:MessageDetails = PopUpManager.createPopUp(this, MessageDetails, true) as MessageDetails;
			details.message = message;
			details.addEventListener(CloseEvent.CLOSE, messageCloseHandler);
			PopUpManager.centerPopUp(details);
			
			//remember the particle we clicked
			this.lastTouched = DisplayObject(event.currentTarget);
		}
		
		/**
		 * @private
		 * Resumes the particle emitter.
		 */
		protected function messageCloseHandler(event:CloseEvent):void
		{
			PopUpManager.removePopUp(MessageDetails(event.currentTarget));
			
			//move to bottom because the user will want to read different messages
			this.lastTouched.parent.setChildIndex(this.lastTouched, 0);
			
			this.emitter.resume();
		}
		
		/**
		 * @private
		 * Bring a particle to the top and stops it from moving when the mouse
		 * is over it. Shows the data tip.
		 */
		protected function particleRollOverHandler(event:MouseEvent):void
		{
			var particle:MessageParticle = MessageParticle(event.currentTarget);
			this.validateNow(); //make sure we have the right size!
			particle.addEventListener(MouseEvent.ROLL_OUT, particleRollOutHandler);
			this.moveAction.addException(particle);
			
			//move to top so that no other particles steal focus
			particle.parent.setChildIndex(particle, particle.parent.numChildren - 1);
			
			this.dataTip.message = particle.message;
			PopUpManager.addPopUp(this.dataTip, this);
			var position:Point = particle.parent.localToGlobal(new Point(particle.x, particle.y));
			this.dataTip.move(position.x + particle.width / 2 + 6, position.y - this.dataTip.height / 2);
		}
		
		/**
		 * @private
		 * Removes the data tip and allows the particle to move again.
		 */
		protected function particleRollOutHandler(event:MouseEvent):void
		{
			var particle:MessageParticle = MessageParticle(event.currentTarget);
			this.validateNow(); //make sure we have the right size!
			particle.removeEventListener(MouseEvent.ROLL_OUT, particleRollOutHandler);
			this.moveAction.removeException(particle);
			
			this.dataTip.message = null;
			PopUpManager.removePopUp(this.dataTip);
		}
		
	}
}